package com.ruoyi.common.utils.collection;

import com.ruoyi.common.utils.StringUtils;

import java.math.BigDecimal;
import java.time.LocalDate;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * @author wjh
 * 2024/4/29
 */
public class CollectionUtils extends org.springframework.util.CollectionUtils {

    /**
     * 是否为空，包括空集合、空元素
     * @param list
     * @return
     */
    public static boolean isEmptyElement(List<?> list) {
        if (list == null) {
            return true;
        }
        // 过滤空元素
        List<?> collect = list.stream().filter(Objects::nonNull).collect(Collectors.toList());
        return org.springframework.util.CollectionUtils.isEmpty(collect);
    }

    public static boolean isNotEmptyElement(List<?> list) {
        return !isEmptyElement(list);
    }

    public static boolean isNotEmpty(List<?> list) {
        return !isEmpty(list);
    }

    /**
     * 判断分隔字符串中是否存在目标元素
     * @param str 分隔字符串
     * @param split 分隔符
     * @param target 目标元素
     * @param parseMapper 类型转换方法
     */
    public static <T> boolean splitContains(String str, String split, T target, Function<String, ? extends T> parseMapper) {
        if (!StringUtils.hasText(str)) {
            return false;
        }
        return Arrays.stream(str.split(split)).map(parseMapper).collect(Collectors.toList()).contains(target);
    }

    /**
     * 集合映射，去除空值、重复值
     */
    public static <T, R> List<R> map(List<T> list, Function<? super T, ? extends R> keyMapper) {
        return list.stream().map(keyMapper).filter(Objects::nonNull).distinct().collect(Collectors.toList());
    }

    /**
     * 累加 BigDecimal
     */
    public static <T> BigDecimal sumBigDecimal(List<? extends T> list, Function<T, BigDecimal> valueMapper) {
        if (CollectionUtils.isEmptyElement(list)) {
            return BigDecimal.ZERO;
        }
        return list.stream().filter(Objects::nonNull)
                .map(valueMapper)
                .filter(Objects::nonNull)
                .reduce(BigDecimal.ZERO, BigDecimal::add);
    }

    /**
     * 累加 Integer
     */
    public static <T> Integer sumInteger(List<? extends T> list,  Function<T, Integer> valueMapper) {
        if (CollectionUtils.isEmptyElement(list)) {
            return 0;
        }
        return list.stream().filter(Objects::nonNull)
                .map(valueMapper)
                .filter(Objects::nonNull)
                .reduce(Integer::sum)
                .orElse(0);
    }

    public static <T> void addNotNull(List<? super T> list, T obj) {
        if (obj == null) {
            return;
        }
        list.add(obj);
    }
}
